@@ -18,7 +18,7 @@
 */
 
 /*
- * $Id: ccid_usb.c 6760 2013-10-01 12:57:50Z rousseau $
+ * $Id: ccid_usb.c 6793 2013-11-25 13:09:41Z rousseau $
  */
 
 #define __CCID_USB__
@@ -447,9 +447,12 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 						/* we reuse the same device
 						 * and the reader is multi-slot */
 						usbDevice[reader_index] = usbDevice[previous_reader_index];
-						/* the other slots do not have the same data rates */
+						/* The other slots of GemCore SIM Pro firmware
+						 * 1.0 do not have the same data rates.
+						 * Firmware 2.0 do not have this limitation */
 						if ((GEMCOREPOSPRO == readerID)
-							|| (GEMCORESIMPRO == readerID))
+							|| ((GEMCORESIMPRO == readerID)
+							&& (usbDevice[reader_index].ccid.IFD_bcdDevice < 0x0200)))
 						{
 							usbDevice[reader_index].ccid.arrayOfSupportedDataRates = SerialCustomDataRates;
 							usbDevice[reader_index].ccid.dwMaxDataRate = 125000;
@@ -494,35 +497,45 @@ status_t OpenUSBByName(unsigned int reader_index, /*@null@*/ char *device)
 				}
 
 again:
-#ifdef __APPLE__
-				/* Some early Gemalto Ezio CB+ readers have
-				 * bDeviceClass, bDeviceSubClass and bDeviceProtocol set
-				 * to 0xFF (proprietary) instead of 0x00.
-				 *
-				 * So on Mac OS X the reader configuration is not done
-				 * by the OS/kernel and we do it ourself.
-				 */
-				if (GEMALTO_EZIO_CBP == readerID)
-				{
-					r = libusb_set_configuration(dev_handle, 1);
-					if (r < 0)
-					{
-						(void)libusb_close(dev_handle);
-						DEBUG_CRITICAL4("Can't set configuration on %d/%d: %d",
-							bus_number, device_address, r);
-						continue;
-					}
-				}
-#endif
-
 				r = libusb_get_active_config_descriptor(dev, &config_desc);
 				if (r < 0)
 				{
-					(void)libusb_close(dev_handle);
-					DEBUG_CRITICAL4("Can't get config descriptor on %d/%d: %d",
-						bus_number, device_address, r);
-					continue;
+#ifdef __APPLE__
+					/* Some early Gemalto Ezio CB+ readers have
+					 * bDeviceClass, bDeviceSubClass and bDeviceProtocol set
+					 * to 0xFF (proprietary) instead of 0x00.
+					 *
+					 * So on Mac OS X the reader configuration is not done
+					 * by the OS/kernel and we do it ourself.
+					 */
+					if ((0xFF == desc.bDeviceClass)
+						&& (0xFF == desc.bDeviceSubClass)
+						&& (0xFF == desc.bDeviceProtocol))
+					{
+						r = libusb_set_configuration(dev_handle, 1);
+						if (r < 0)
+						{
+							(void)libusb_close(dev_handle);
+							DEBUG_CRITICAL4("Can't set configuration on %d/%d: %d",
+									bus_number, device_address, r);
+							continue;
+						}
+					}
+
+					/* recall */
+					r = libusb_get_active_config_descriptor(dev, &config_desc);
+					if (r < 0)
+					{
+#endif
+						(void)libusb_close(dev_handle);
+						DEBUG_CRITICAL4("Can't get config descriptor on %d/%d: %d",
+							bus_number, device_address, r);
+						continue;
+					}
+#ifdef __APPLE__
 				}
+#endif
+
 
 				usb_interface = get_ccid_usb_interface(config_desc, &num);
 				if (usb_interface == NULL)
